/** Under Posix we use the normal stdio, set to non-blocking cannonical mode **/
#include <signal.h>
#include <termios.h>
#include <unistd.h>
#include <stdio.h>
#include "Errors.h"

struct termios tios, orig_tios;

void StdioDestroy() {
  fprintf(stderr,"DEBUG: StdioDestroy()\n");
  /* Restore terminal settings */
  tcsetattr(0, TCSANOW, &orig_tios);
}

void StdioInit() {
  struct sigaction saio;           /* definition of signal action */

  /* install the signal handler before making the device asynchronous */
  saio.sa_handler = StdioDestroy;
  sigemptyset(&saio.sa_mask);
  saio.sa_flags = 0;
  saio.sa_restorer = NULL;
  sigaction(SIGIO,&saio,NULL);

  /* Put terminal in immediate input mode */
  if (tcgetattr(0, &orig_tios))
    ERRORreturn("Error getting tios terminal settings");

  /* copy that to tios and play with it */
  tios = orig_tios;

  /* We want to disable the canonical mode */
  tios.c_lflag &= ~ICANON;

  /* And make sure ECHO is disabled */
  tios.c_lflag &= ~ECHO;

  /* Apply our settings */
  if (tcsetattr(0, TCSANOW, &tios))
    ERRORreturn("Error applying terminal settings");

  /* Check whether our settings were correctly applied */
  if (tcgetattr(0, &tios)){
    tcsetattr(0, TCSANOW, &orig_tios);
    ERRORreturn("Error while asserting terminal settings");
  }

  if ((tios.c_lflag & ICANON) || (tios.c_lflag & ECHO)) {
    tcsetattr(0, TCSANOW, &orig_tios);
    ERRORreturn("Could not apply all terminal settings");
  }
}

