#include "Platform.h"
#include "Terminal.h"

/* Circular buffer */
unsigned char termBuffer[TERM_HEIGHT][TERM_WIDTH];
/* Attributes should be bit-packed **/
unsigned char termAttrib[TERM_HEIGHT][TERM_WIDTH];
/* Circular buffer origin - allows for easy scrolling */
int termOffsetX = 0;
int termOffsetY = 0;
/* cursor location */
int cursorX = 0;
int cursorY = 0;
uint8_t cursorAttr = 0; // cursor attributes

void TerminalInit() {
  Clear();
}

void Put(char c) { 
  if (c == '\n') { // special newline handling
    cursorX = 0; // back to start of line
  } else {
    getTermBuffer(cursorY, cursorX) = c; 
    getAttrib(cursorY, cursorX) = cursorAttr;
    cursorX = (cursorX + 1) % TERM_WIDTH;
  }
  if (!cursorX) { // new line
    if (cursorY == TERM_HEIGHT - 1) // bottom line already, then scroll
      termOffsetY = mod(termOffsetY + 1, TERM_HEIGHT);
    else
      cursorY++;
  }
  VideoUpdate();
}

void Puts(const char *s) { 
  for (const char *p=s; *p; ++p)
    Put(*p);
}
void SetBright() { cursorAttr |= BoldFlag; }
void UnsetBright() { cursorAttr &= ~BoldFlag; }
void SetInvert() { cursorAttr |= InvertFlag; }
void UnsetInvert() { cursorAttr &= ~InvertFlag; }
void SetNormal() { UnsetBright(); UnsetInvert(); }
void SetPos(int x, int y) { 
  cursorX = mod(x, TERM_WIDTH);
  cursorY = mod(y, TERM_HEIGHT);
}

void Clear() {
  for (int i=0; i<TERM_HEIGHT; ++i) {
    for (int j=0; j<TERM_WIDTH; ++j) {
#if 1
      termBuffer[0][0] = 0; // no need for circular calcs here
#else // test pattern
      termBuffer[i][j] = '0' + ((i+j) % 60);
#endif
      termAttrib[i][j] = 0;
    }
  }
  VideoUpdate();
}

/* Terminal Interface */
#if 0
void Put(char c) { putchar(c); }

void Puts(const char *s) { puts(s); }
void SetBright() { printf("\033[1m"); }
void UnsetBright() { printf("\33[22m"); }
void SetInvert() { printf("\33[7m"); }
void UnsetInvert() { printf("\33[27m"); }
void SetNormal() { UnsetBright(); UnsetInvert(); }
void SetPos(int x, int y) { printf("\33[%i;%iH", y, x); }
void Clear() { printf("\033[2J"); }
#endif
