/** By Braddock Gaskill, 8/9/09 **/

#ifndef PLATFORM_HPP_
#define PLATFORM_HPP_ 1

#include <SDL/SDL.h>
#include <unistd.h>
#include "Errors.h"
#include "Platform.h"
#include "Terminal.h"

SDL_Surface *sdlScreen;
SDL_Event sdlEvent;

/* Uzebox Mode 1 */
//define WIDTH 240
#define WIDTH 320
#define HEIGHT 224
#define DEPTH 24

#ifndef PROGMEM
#  define PROGMEM
#endif

#if 0
#include "data/fonts_8x8_2.pic.inc"
#else
#include "data/iso8859-1.c"
#endif

#define SDL_MULT 2
const int SDL_WIDTH=SDL_MULT*WIDTH;
const int SDL_HEIGHT=SDL_MULT*HEIGHT;

void putpixel(SDL_Surface *surface, int x, int y, Uint32 pixel) {
  int bpp = surface->format->BytesPerPixel;
  /* Here p is the address to the pixel we want to set */
  Uint8 *p = (Uint8 *)surface->pixels + y * surface->pitch + x * bpp;

  switch(bpp) {
    case 1:
      *p = pixel;
      break;

    case 2:
      *(Uint16 *)p = pixel;
      break;

    case 3:
      if(SDL_BYTEORDER == SDL_BIG_ENDIAN) {
        p[0] = (pixel >> 16) & 0xff;
        p[1] = (pixel >> 8) & 0xff;
        p[2] = pixel & 0xff;
      } else {
        p[0] = pixel & 0xff;
        p[1] = (pixel >> 8) & 0xff;
        p[2] = (pixel >> 16) & 0xff;
      }
      break;

    case 4:
      *(Uint32 *)p = pixel;
      break;
  }
}

void putpixelscale(SDL_Surface *surface, int x, int y, Uint32 pixel, int scale) {
  assert(x >= 0 && x < WIDTH);
  assert(y >= 0 && y < HEIGHT);
  for (int i=0; i<scale; ++i)
    for (int j=0; j<scale; ++j)
      putpixel(surface, scale*x + i, scale*y + j, pixel);
}

void VideoInit() {
  DebugPrint("VideoInit\n");
  if (SDL_Init(SDL_INIT_VIDEO) < 0 ) 
    ERRORreturn("SDL_Init Failed");
  if (!(sdlScreen = SDL_SetVideoMode(SDL_WIDTH, SDL_HEIGHT, DEPTH, 0x0 /*| SDL_HWSURFACE | SDL_DOUBLEBUF*/))) {
    SDL_Quit();
    ERRORreturn("SDL_SetVideoMode failed");
  }
  SDL_WM_SetCaption( "Tviki", NULL ); 
}


/** Text Display 
 * - we assume a simple tile-based implementation with text attributes **/

void ScrClearNoFlip() {
  for (int x = 0; x < sdlScreen->w; ++x) {
    for (int y = 0; y < sdlScreen->h; ++y) {
      /* Kinda classic X11 screen... :) */
      //putpixel(sdlScreen, x, y, 0xffffff * ((x + y)%2));
      putpixel(sdlScreen, x, y, 0x0);
    }
  }
}

void ScrClear() {
  //If the surface must be locked 
  if( SDL_MUSTLOCK( sdlScreen ) ) //Lock the surface 
    SDL_LockSurface( sdlScreen ); 
  ScrClearNoFlip();
  if( SDL_MUSTLOCK( sdlScreen ) ) //Lock the surface 
    SDL_UnlockSurface( sdlScreen ); 
  SDL_Flip(sdlScreen);
}

void BlitByteToBitArray(uint8_t b, int bpos, unsigned char *bits) {
  int base = bpos / 8;
  int shift = bpos % 8;
  bits[base] |= (b >> shift);
  if (shift)
    bits[base+1] |= b << (8 - shift);
}

const unsigned char *GetGlyphRow(const unsigned char *font, int glyphWidth, int glyphHeight, int glyphModulo, int idx, int row) {
  return font + idx * glyphWidth * (glyphHeight+glyphModulo) / 8 + row*(glyphWidth/8 + glyphModulo/8);
}

/** Render a line from the terminal tile buffer into two bit-plane line buffers using font.
 *  Assumptions: Font is 8-bits wide. **/
void RenderLineToBuffer(
    unsigned char *nextbits0, /* bit-plane lines */
    int termW, /* terminal buffer width in tiles, modulo offset for buffer */
    int remainderY, /* row offset into glyphs for this line */
    unsigned char *termLine, unsigned char *attribLine, /* pointers to line of tiles and bold/invert attributes */
    uint8_t glyphWidth, uint8_t glyphHeight, uint8_t glyphModulo, const unsigned char *font) /* font information */
{
  for (int i=0; i < termW; ++i) {
    uint8_t c = termLine[i];
    const unsigned char *glyph = GetGlyphRow(font, glyphWidth, glyphHeight, glyphModulo, c, remainderY);
    BlitByteToBitArray(*glyph, i * glyphWidth, nextbits0);
  }
}

void VideoUpdate() {
  unsigned char linebits0[WIDTH/8];
  if( SDL_MUSTLOCK( sdlScreen ) ) //Lock the surface 
    SDL_LockSurface( sdlScreen ); 
  ScrClearNoFlip();
  for (int j=0; j<TERM_HEIGHT; ++j) {
    for (int rem=0; rem < FONT_TILESET_HEIGHT; ++rem ) {
#if 1
      for (int i=0; i<WIDTH/8; ++i) linebits0[i] = 0x0;
      const int fontModulo = 0;
      RenderLineToBuffer(linebits0, TERM_WIDTH, rem, &getTermBuffer(j,0), &getAttrib(j,0), 
          FONT_TILESET_WIDTH, FONT_TILESET_HEIGHT, fontModulo, font_bitpacked);
      for (int i=0; i<WIDTH/8; ++i) {
        unsigned char c = linebits0[i];
        for (int ii=0; ii < 8; ++ii) {
          unsigned char cpix = (c >> ii) & 0x1;
          putpixelscale(sdlScreen, i*8 + (7-ii), rem + j*FONT_TILESET_HEIGHT, cpix ? 0xff : 0x30, SDL_MULT);
        }
      }
#endif
    }
  }
  if( SDL_MUSTLOCK( sdlScreen ) ) //Lock the surface 
    SDL_UnlockSurface( sdlScreen ); 
  SDL_Flip(sdlScreen);
}

#if 0
void VideoUpdate() {
  //const unsigned int buflen = RenderBufferSize();
  const unsigned int buflen = PIXEL_WIDTH * 3 * CELL_Y_ADVANCE;
  unsigned char buf[buflen];
  memset(buf, buflen, 0x0);
  ScrClear();
  if( SDL_MUSTLOCK( sdlScreen ) ) //Lock the surface 
    SDL_LockSurface( sdlScreen ); 
  for (int row=-1; row < CELLMAP_HEIGHT; ++row) {
    RenderRow(buf, row);
    if (row == -1)
      continue;
    /* Copy row worth of rendering to buffer */
    for (int i=0; i< PIXEL_WIDTH; ++i) {
      for (int j=0; j<CELL_Y_ADVANCE; ++j) {
        //unsigned char v = buf[i + j*RenderBufferWidth()];
        unsigned char v = RenderBufferGet(buf, i, row*CELL_Y_ADVANCE + j);
        putpixelscale(sdlScreen, i, row*CELL_Y_ADVANCE + j, v, SDL_MULT);
        RenderBufferSet(buf, i, row*CELL_Y_ADVANCE + j, 0);
      }
    }
  }
  if( SDL_MUSTLOCK( sdlScreen ) ) //Lock the surface 
    SDL_UnlockSurface( sdlScreen ); 
  SDL_Flip(sdlScreen);
}
#endif

void InputInit() { 
  SDL_EnableUNICODE(1);
}

char ReadKey(KeyEvent *outEvent) {
  SDL_Event event;

  while (!SDL_PollEvent(&event)) {
    switch(event.type) {
      case SDL_KEYDOWN:
      case SDL_KEYUP:
        outEvent->isRelease = (event.type == SDL_KEYDOWN);
        outEvent->scancode = event.key.keysym.scancode;
        outEvent->keysym = event.key.keysym.sym;
        outEvent->ucs4 = event.key.keysym.unicode;
        return 1;
        break;
      default:
        return 0;
    }
  }
  return 0;
}
#endif //PLATFORM_HPP_
