#include <avr/io.h>
#include "z810-defines.h"

#include "../uCmorse/morse_asm.h"

.global TIMER1_COMPA_vect
.global SetColorBurstOffset
.global burstOffset
.global vsync_phase
.global curr_frame
.global sync_pulse
.global sync_phase
.global curr_field
.global run_timetable
.global test_delay

#ifndef F_CPU
#  define F_CPU 28636360
#endif
#define F_CPU_MS (F_CPU/1000)

#define nsec2cyc(n) ((n*F_CPU_MS)/1000000)

#define T_HSYNC  nsec2cyc(4700)
#define T_FPORCH nsec2cyc(1500)
#define T_BPORCH nsec2cyc(4700)
#define T_VIDEO  nsec2cyc(52600)
#define T_LINE (T_FPORCH + T_BPORCH + T_HSYNC + T_VIDEO)
#define T_HALF_LINE (T_LINE/2)

#define FIX1 nsec2cyc(1600)
//define FIX2 nsec2cyc(3600)
//define FIX1 (0)
#define FIX2 (25)

.section .data

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Time Table
;; DELAY, FUNCTION, ARG1, ARG2
timetable:
;
; FIELD 0
; pre-eq phase
.word T_HSYNC - FIX1                  , pm(sync_low)    $ .byte 0, 0
.word T_HALF_LINE - T_HSYNC - FIX2    , pm(sync_high)   $ .byte 0, 0
.word 0                               , pm(counted_loop)$ .byte 2, 6
; vert sync/eq pulses
.word T_HALF_LINE - T_HSYNC - FIX2    , pm(sync_low)    $ .byte 0, 0
.word T_HSYNC - FIX1                  , pm(sync_high)   $ .byte 0, 0
.word 0                               , pm(counted_loop)$ .byte 2, 6
; post-eq pulses
.word T_HSYNC - FIX1                  , pm(sync_low)    $ .byte 0, 0
.word T_HALF_LINE - T_HSYNC - FIX2    , pm(sync_high)   $ .byte 0, 0
.word 0                               , pm(counted_loop)$ .byte 2, 6
; after this, normal field1 hsync lines start
.word T_HSYNC - FIX1                  , pm(sync_low)    $ .byte 0, 0
.word T_BPORCH - FIX1                 , pm(sync_high)   $ .byte 0, 0
.word T_VIDEO + T_FPORCH - FIX2       , pm(render_line) $ .byte 0, 0
.word 0                               , pm(counted_loop)$ .byte 3, 253
; half line
.word T_HSYNC - FIX1                  , pm(sync_low)    $ .byte 0, 0
.word T_HALF_LINE - T_HSYNC - FIX2, pm(sync_high)   $ .byte 0, 0
; finished field
;
; FIELD 1
; pre-eq phase
.word T_HSYNC - FIX1                  , pm(sync_low)    $ .byte 0, 0
.word T_HALF_LINE - T_HSYNC - FIX2    , pm(sync_high)   $ .byte 0, 0
.word 0                               , pm(counted_loop)$ .byte 2, 6
; vert sync/eq pulses
.word T_HALF_LINE - T_HSYNC - FIX2    , pm(sync_low)    $ .byte 0, 0
.word T_HSYNC - FIX1                  , pm(sync_high)   $ .byte 0, 0
.word 0                               , pm(counted_loop)$ .byte 2, 6
; post-eq pulses - 7 in this field
.word T_HSYNC - FIX1                  , pm(sync_low)    $ .byte 0, 0
.word T_HALF_LINE - T_HSYNC - FIX2    , pm(sync_high)   $ .byte 0, 0
.word 0                               , pm(counted_loop)$ .byte 2, 7
; after this, normal field1 hsync lines start
.word T_HSYNC - FIX1                  , pm(sync_low)    $ .byte 0, 0
.word T_BPORCH - FIX1                 , pm(sync_high)   $ .byte 0, 0
.word T_VIDEO + T_FPORCH - FIX1       , pm(render_line) $ .byte 0, 0
.word 0                               , pm(counted_loop)$ .byte 3, 253
; All done!  Restart.
.word 0                               , pm(restart_timetable) $ .byte 0, 0

.word 0x0200          , pm(sync_high)    $ .byte 0, 0
.word 0x0100          , pm(sync_low)    $ .byte 0, 0
.word 0x0200          , pm(sync_high)    $ .byte 0, 0
.word 0x0100          , pm(sync_low)    $ .byte 0, 0
.word 0x0200          , pm(sync_high)    $ .byte 0, 0
.word 0x0100          , pm(sync_low)    $ .byte 0, 0
;.word 0                         , pm(counted_loop)$ .byte 2, 2
.word 0                         , pm(restart_timetable) $ .byte 0, 0


;; DEBUG TIMETABLE
.word 0x0640          , pm(sync_toggle)    $ .byte 0, 0
.word 0x1000          , pm(sync_toggle)    $ .byte 0, 0
.word 0                         , pm(counted_loop)$ .byte 2, 3
.word 0x2000          , pm(sync_toggle)    $ .byte 0, 0
.word 0x3000          , pm(sync_toggle)   $ .byte 0, 0
.word 0                         , pm(counted_loop)$ .byte 2, 2
.word 0x4000          , pm(sync_toggle)    $ .byte 0, 0
.word 0                         , pm(counted_loop)$ .byte 1, 2
.word 0x5000          , pm(sync_toggle)   $ .byte 0, 0
.word 0                         , pm(counted_loop)$ .byte 1, 2
.word 0               , pm(restart_timetable) $ .byte 0, 0
;;;;






;;; State for interrupt-driven timetable func
tt_cur_ptr: ; Ptr to current timetable entry
.dc.w timetable
tt_target_time: ; Target clock time
.dc.w 0x0
tt_cur_repeat: ; repeat counter
.dc.b 0x0,0x0

#define tt_arg1 r23
#define tt_arg2 r24

.macro push_regs_r21_to_Z
  push YH
  push YL
  push ZH
	push ZL 
	;save flags & status register
	in ZL,_SFR_IO_ADDR(SREG)
  push ZL
  push XL
  push XH
  push r21
  push r22
  push r23
  push r24
  push r1
  push r2
.endm

.macro pop_regs_r21_to_Z
  pop r2
  pop r1
  pop r21
  pop r22
  pop r23
  pop r24
  pop XH
  pop XL
	pop ZL
	out _SFR_IO_ADDR(SREG),ZL
  pop ZL
  pop ZH
  pop YL
  pop YH
.endm
.section .text

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; busy-wait 16-bit delay by X cycles inc call.  
;;; X can be any value 0 - 2**16
;;; Resolution is 4 cycles
;;; DESTROYS X
;;; Actual Scope Measurements F_CPU=28,636,360
;;; nop, 100ns (inc sbi?),  32=1.1us
;;; 100=3.5us 400=14us 1600=55us 6400=225us 25600=900us 
.macro delay
91:     ; loop is 4 per iteration - 1 at exit
    sbiw XL,4   ;2
    brcc 91b ;1/2
.endm

;;;;;;;;;;;;;;;;;;;
;; RULES FOR TIMETABLE ROW FUNCTIONS
;; X = current row delay
;; Y = tt_cur_ptr
;; Z = scratch
;; r0,r1,r22,r23,r24,r25 = scratch, use as needed
;; call tt_load_args IF NEEDED => r23,r24
run_timetable_row:
#ifdef DOMORSE
; DEBUG: Get Row Number
lds YL, tt_cur_ptr+0
lds YH, tt_cur_ptr+1
ldi XL, lo8(timetable)
ldi XH, hi8(timetable)
sub YH, XH
sbc YL, XL
lsr YL
lsr YL
repeatTapCharCI 50, 'R', YL
#endif

  ; Get Row 
  lds YL, tt_cur_ptr+0
  lds YH, tt_cur_ptr+1

  ; Store Row Delay
  ldd XL, Y+0
  ldd XH, Y+1

  ; Get Row Func
  ldd ZL, Y+2
  ldd ZH, Y+3

  ; Call Row Func
  icall  ; (Z)

  ; Increment and Store Row Pointer (may have changed)
  adiw YL, 6
  sts tt_cur_ptr+0, YL
  sts tt_cur_ptr+1, YH
  
  ; Return Delay in X
  ; Cleanup
  ret

.section .data
debug_cnt:
.word 0

.section .text

run_timetable:
  push_regs_r21_to_Z
  sbi _SFR_IO_ADDR(DDRA),PA1
  sbi _SFR_IO_ADDR(DDRA),PA3
  sbi _SFR_IO_ADDR(DDRB),SYNC_PIN
;;setMorseSpeedScale 1
rtt_loop:
  call run_timetable_row
  ;; delay returned in X
repeatTapCharCI 70, 'D', XH
  ;cpi XL,0
  ;cpc XH,0
  ;breq rtt_loop ; delay == 0 - unnecessary?
  ;sbi _SFR_IO_ADDR(PINA),PA3
  delay 
  ;inc debug_cnt
  
  ;sbi _SFR_IO_ADDR(PINA),PA3
  jmp rtt_loop
  pop_regs_r21_to_Z
  ret

;;; TEST CASE
;TIMER1_COMPA_vect:
  push_regs_r21_to_Z ;;; optimize me
  sbi _SFR_IO_ADDR(DDRA),PA1
  sbi _SFR_IO_ADDR(DDRA),PA3
  sbi _SFR_IO_ADDR(DDRB),SYNC_PIN
 
  sbi _SFR_IO_ADDR(PINA), PA3
 
  lds XL, _SFR_MEM_ADDR(OCR1AL)
  lds XH, _SFR_MEM_ADDR(OCR1AH)

  ;ldi r21,0x10
  ;add XH,r21
  ;andi XH,0xf0
  ldi r21,0x01
  add XH,r21
  andi XH,0x0f
  brne 1f
    sbi _SFR_IO_ADDR(PINA), PA1;
    ldi XH, 0x01 ; don't want 0 == 65k
1:

  ;;; Set timer with new delay
  sts _SFR_MEM_ADDR(OCR1AH), XH
  sts _SFR_MEM_ADDR(OCR1AL), XL
 
  pop_regs_r21_to_Z
  reti  


TIMER1_COMPA_vect:
  push_regs_r21_to_Z ;;; optimize me
  sbi _SFR_IO_ADDR(DDRA),PA1
  sbi _SFR_IO_ADDR(DDRA),PA3
  sbi _SFR_IO_ADDR(DDRB),SYNC_PIN

run_loop: 
  call run_timetable_row

  ; Don't return on small intervals, just loop
  ldi r22, 0x0
  ldi r23, 0xff
  cp  XL, r23
  cpc XH, r22
  brsh 9f
    lds r23, _SFR_MEM_ADDR(TCNT1L)
    lds r22, _SFR_MEM_ADDR(TCNT1H)
    sub XL, r23
    sbc XH, r22
    brcs 8f ; We're late!  No delay!
      ;sbi _SFR_IO_ADDR(PINA),PA3
      delay 
      ;sbi _SFR_IO_ADDR(PINA),PA3
8:
    ;reset timer as if interrupt happened
    clr r21
    sts _SFR_MEM_ADDR(TCNT1H), r21
    sts _SFR_MEM_ADDR(TCNT1L), r21
    jmp run_loop
9:

  ;ldi  r21, (1 << CS10) | (1 << WGM12)
  ;sts _SFR_MEM_ADDR(TCCR1B), r21

  ; We subtract a hack for INT overhead compensation
  ldi r22, 0
  ldi r23, 35 
  sub XL,r23
  sbc XH,r22
  sts _SFR_MEM_ADDR(OCR1AH), XH
  sts _SFR_MEM_ADDR(OCR1AL), XL
 
  pop_regs_r21_to_Z
  reti

sync_low:
sbi _SFR_IO_ADDR(DDRA),PA3
  sbi _SFR_IO_ADDR(DDRB),SYNC_PIN

cbi _SFR_IO_ADDR(PORTA),PA1 ;; just to clear restart signal
cbi _SFR_IO_ADDR(PORTA),PA3
  cbi _SFR_IO_ADDR(PORTB),SYNC_PIN
  ret

sync_high:
sbi _SFR_IO_ADDR(PORTA),PA3
  sbi _SFR_IO_ADDR(PORTB),SYNC_PIN
  ret

sync_toggle:
sbi _SFR_IO_ADDR(PINA),PA3
  sbi _SFR_IO_ADDR(PINB),SYNC_PIN
  ret

counted_loop:
  lds r22, tt_cur_repeat
  ldd r23, Y+4  ; backwards skip
  ldd r24, Y+5  ; repeat count

  ; Do we need to start our loop?
  cpi r22, 0
  brne not_eq_zero_cl
    mov r22, r24 ; load the repeat count
  not_eq_zero_cl:

  ; Did our loop just finish?
  ; Note, loop repeat of 1 is done without looping!
  cpi r22, 1
  brne not_eq_one_cl
    dec r22 ; last pass of loop now finished
    sts tt_cur_repeat, r22
    rjmp cleanup_cl
  not_eq_one_cl:

  ;; loop already started or just started
  dec r22  ; dec Repeat Count
  sts tt_cur_repeat, r22  ; Store repeat count

  ; Relocate Row Pointer
  inc r23 ; account for table increment
  ldi r25,6
  mul r23,r25
  sub YL, r0 ; move back row pointer
  sbc YH, r1 ; move back row pointer
cleanup_cl:
  ret

render_line:
  call render_bg_line
  ret

restart_timetable:
  sbi _SFR_IO_ADDR(DDRA),PA1
  sbi _SFR_IO_ADDR(PORTA),PA1
  ;call syncPulseA1
  ldi YL, lo8(timetable)
  ldi YH, hi8(timetable)
  sbiw YL, 6
  ret

render_bg_line:
  push r16
  push r17

  ldi r16,232
bg_line_loop:
  nop
  nop
  nop
  out _SFR_IO_ADDR(DATA_PORT),r16
  dec r16
  brne bg_line_loop
  
  pop r17
  pop r16
  ret

.macro jitter_fix9 tdiff
  ; Use the fact that brlo uses either 1 or 2 
  ; cycles based on comparison to un-jitter timing
	cpi \tdiff,1
	brlo .		;advance PC to next instruction
	cpi \tdiff,2
	brlo .		;advance PC to next instruction
	cpi \tdiff,3
	brlo .		;advance PC to next instruction
 	cpi \tdiff,4
	brlo .		;advance PC to next instruction
	cpi \tdiff,5
	brlo .		;advance PC to next instruction
	cpi \tdiff,6
	brlo .		;advance PC to next instruction
	cpi \tdiff,7
	brlo .		;advance PC to next instruction
	cpi \tdiff,8
	brlo .		;advance PC to next instruction
	cpi \tdiff,9
	brlo .		;advance PC to next instruction
.endm

test_delay:
  call syncPulseA1

  sbi _SFR_IO_ADDR(DDRA),PA3
  sbi _SFR_IO_ADDR(DDRA),PA1

  sbi _SFR_IO_ADDR(PORTA),PA3
  nop
  sbi _SFR_IO_ADDR(PINA),PA3

  ldi XH, 0
  ldi XL, 0x20
  delay

  sbi _SFR_IO_ADDR(PINA),PA3
  ldi XH, 0
  ldi XL, 0x64
  delay
  sbi _SFR_IO_ADDR(PINA),PA3
  
  lsl XL
  rol XH
  lsl XL
  rol XH
  delay
  sbi _SFR_IO_ADDR(PINA),PA3

  lsl XL
  rol XH
  lsl XL
  rol XH
  delay
  sbi _SFR_IO_ADDR(PINA),PA3

  lsl XL
  rol XH
  lsl XL
  rol XH
  delay
  cbi _SFR_IO_ADDR(PORTA),PA3

  lsl XL
  rol XH
  lsl XL
  rol XH
  delay
  cbi _SFR_IO_ADDR(PORTA),PA3

  ; Should be near overflow limit now

  ret





