#include <string.h>
#include "vt52.h"
#include "Platform.h"
#if 1
#include "crc32.h"
#include "bgApi.h"
#include "bgUtil.h"
#include "bgUtil2.h"
#include "uCmorse.h"
#endif
#ifdef AVR
#  include <avr/io.h>
#endif

typedef struct {
  enum {
    CategoryList,
    ArticleList,
    SectionList,
    LinkList
  } type;
  Cursor listStart; //< Cursor pointing to start of list
  Cursor listPos; //< Cursor pointing to selected element
  int offset;
} HistoryItem;

#define HistoryStackSize 8

typedef struct {
  DbContext pageContext;
  BgTree pageTree;
  HistoryItem historyStack[HistoryStackSize];
  int historyMax; /**< Size of history stack **/
} WikiApp;

void WikiAppInit(WikiApp *wa) {
  wa->pageContext.tableFname = "zb24.bgt";
  wa->pageContext.blobFname  = "zb24.bgb";
  wa->pageContext.fileStateVersion = 0;
  TreeInit(&wa->pageTree, &wa->pageContext);
}

Cursor WikiAppTitleSearch(WikiApp *wa, const char *title) {
  /* Hash title string */
  KeyT key = CalcCRC(title, strlen(title));
  /* Perform search for title hash */
  printf("\nSearching...\n");
  Cursor c = SelectConsecutiveKeys(wa->pageTree, key);
  printf("Search Complete\n");
  return c;
}

// diskfilename parameter defined in patfs-stdio
const char *diskfilename;

typedef struct {
  char inBuf[32];
  int posBuf;
} App;

void AppInit(App *app) {
  app->inBuf[0] = 0x0;
  app->posBuf = 0;
}

#if 0
void renderList(HistoryItem *hi) {
  Cursor c = CursorDup(&hi->listStart);
  for (uint32_t found = CursorNext(&c); found != BAInvalid; found = CursorNext(&c)) {
    FileOpenRO(wa.pageContext.blobFname);
    FileSeek(found);
    // Read xixss blob record
    int32_t level = UnpackIntFromFile(FileGetHandle());
    uint32_t parent = UnpackUIntFromFile(FileGetHandle());
    for (int i=0; i<level; ++i)
      printf("+");
    printf(" ");
    ERRORassert();
    UnpackAndPrintStringFromFile(FileGetHandle(), stdout);
    fprintf(stdout, "\n"); 
    continue;
    uint32_t textSize = UnpackUIntFromFile(FileGetHandle());
    int lastCR=0;
    for (int i=0; i<textSize; ++i, ++lastCR) {
      char buf[32];
      FileRead(buf, 1); // kinda inefficient
      ERRORassert();
      if (buf[0] == ' ' && lastCR > 33) {
        fputc('\n', stdout); // word wrap
        lastCR = 0;
      } else {
        fputc(buf[0], stdout);
      }
    }
  }
}
#endif

void refresh(App *app, WikiApp *wa) {
  printf(VERASE_SCR);
  vposition(stdout, 1, 1);
  printf(VERASE_LINE_CUR);
  printf("Search> %s", app->inBuf);


}

int main(int argc, char ** argv) {
  PlatformInit();
  printf(VERASE_SCR);
  WikiApp wa;
  WikiAppInit(&wa);
  App app;
  AppInit(&app);
#if 1
  { // test only
    FileOpenRO(wa.pageContext.tableFname);
    ERRORassert();
    NodeVisitDFS(wa.pageTree, &TestSanityVisitor, wa.pageTree.topNode);
  }
#endif
  int r=0x0;
  while (1) {
    refresh(&app, &wa);
    r = fgetc(stdin);
    if (r == '\n' || r == '\r') { 
      Cursor c = WikiAppTitleSearch(&wa, app.inBuf);
      vposition(stdout, 3, 1);
      printf(VERASE_DOWN);
      app.posBuf = 0;
      app.inBuf[0] = 0x0;
      for (uint32_t found = CursorNext(&c); found != BAInvalid; found = CursorNext(&c)) {
        FileOpenRO(wa.pageContext.blobFname);
        FileSeek(found);
        // Read xixss blob record
        int32_t level = UnpackIntFromFile(FileGetHandle());
        uint32_t parent = UnpackUIntFromFile(FileGetHandle());
        //printf(" level = %i, parent = %08x\n ", (int) level, (unsigned int) parent);
        for (int i=0; i<level; ++i)
          printf("+");
        printf(" ");
        //ERRORassert();
        UnpackAndPrintStringFromFile(FileGetHandle(), stdout);
        fprintf(stdout, "\n"); 
      }
      app.posBuf=0;
    } else if (r == '\b' || r == 127) {
      app.posBuf > 0 ? app.posBuf-- : 0;
      app.inBuf[app.posBuf] = 0x0;
    } else {
      app.inBuf[app.posBuf++] = r;
      app.inBuf[app.posBuf] = 0x0;
    }
  }
  return 0;
}
