#include <stdio.h>
#include <assert.h>

#include "Hierarchy.h"

int SimpleRender(Level *lev, int index, FILE *outFile) { 
  if (index > 8)
    return -1;
  int sz = 0;
  if (outFile)
    sz = fprintf(outFile, "%i", index*index);
  return sz;
}

Level SimpleExpand(Level *lev, int index) {
  Level sublev = LevelCreate(SimpleRender, lev->expand, 0x0, 0x0);
  return sublev;
}

void Test1() {
  Level lev = LevelCreate(SimpleRender, 0x0, 0x0, 0x0);
  LevelBegin(&lev);
  assert(LevelCanFocus(&lev, 0) == 1);
  assert(LevelCanExpand(&lev, 0) == 0);
  assert(LevelGetFocused(&lev) == -1);
  assert(LevelGetExpanded(&lev) == -1);
  assert(lev.rendered == -1);
  LevelBegin(&lev);
  assert(LevelRender(&lev, 100, 0x0) == -1);
  assert(LevelRender(&lev, 0, 0x0) >= 0);
  int i=0;
  while (1) {
    if (!LevelExists(&lev, i))
      break;
    printf("%i: ", i);
    int res = LevelRender(&lev, i, stdout);
    if (res == -1)
      break;
    printf("\n");
    ++i;
  }
}

/* Test basic Hier interface **/
void Test2() {
  Level lev = LevelCreate(SimpleRender, SimpleExpand, 0x0, 0x0);
  Hier h;
  HierInit(&h, &lev);
  assert(HierGetDepth(&h) == 1);

  HierItem item = HierGetOffset(&h);
  assert(item.level == -1);
  assert(item.item == -1);

  item = HierFirst(&h);
  assert(item.level == 0);
  assert(item.item == 0);
  HierSetOffset(&h, item);
  HierItem item2 = HierGetOffset(&h);
  assert(item.level == item2.level);
  assert(item.item == item2.item);

  HierFocus(&h, item);
  item2 = HierGetFocused(&h);
  assert(item.level == item2.level);
  assert(item.item == item2.item);

  HierFocusNext(&h);
  item2 = HierGetFocused(&h);
  assert(item.level == item2.level);
  assert(item.item != item2.item);

  HierFocusPrev(&h);
  item2 = HierGetFocused(&h);
  assert(item.level == item2.level);
  assert(item.item == item2.item);

  HierExpand(&h, HierFirst(&h));
  assert(HierGetDepth(&h) == 2);

  HierFocus(&h, HierItemCreate(1, 0));
  item = HierGetFocused(&h);
  assert(item.level == 1);
  assert(item.item == 0);

  HierUnexpand(&h, 0);
  assert(HierGetDepth(&h) == 1);
  item = HierGetFocused(&h);
  assert(item.level == 0);
  assert(item.item == 0);
}

void Test3() {
  Level lev = LevelCreate(SimpleRender, SimpleExpand, 0x0, 0x0);
  Hier h;
  HierInit(&h, &lev);
  HierExpand(&h, HierItemCreate(0, 2));
  HierExpand(&h, HierItemCreate(1, 4));
  HierItem it = HierFirst(&h);
  while (1) {
    if (it.item == -1)
      break;
    for (int i=0; i<it.level; ++i)
      printf("  ");
    LevelRender(&h.levels[it.level], it.item, stdout);
    printf("\n");
    if (it.item == LevelGetExpanded(&h.levels[it.level])) {
      it.level = it.level + 1;
      it.item = 0;
    } else {
      it = HierNext(&h, it);
    }
  }
}

int main(int argc, char **argv) {
  Test1();
  Test2();
  Test3();
  return 0;
}
